# Gateway React Component

- [Gateway React Component](#gateway-react-component)
  - [Overview](#overview)
  - [Official Documentation](#official-documentation)
  - [Example Project](#example-project)
  - [Getting started](#getting-started)
    - [1. Install the component](#1-install-the-component)
    - [2. Include the gateway context](#2-include-the-gateway-context)
    - [3. Accessing or requesting the gateway token](#3-accessing-or-requesting-the-gateway-token)
    - [4. IdentityButton behaviour](#4-identitybutton-behaviour)
  - [API Documentation](#api-documentation)
    - [GatewayProvider](#gatewayprovider)
    - [GatewayStatus](#gatewaystatus)
    - [WalletAdapter](#walletadapter)
    - [useGateway](#usegateway)
    - [Client options](#client-options)
    - [Triggering early pass refresh](#triggering-early-pass-refresh)
    - [Paying for your customers' passes](#paying-for-your-customers-passes)
    - [Wrapper](#wrapper)
  - [Contributing](#contributing)
    - [`yarn build`](#yarn-build)
    - [`yarn publish`](#yarn-publish)
    - [Automated publishing](#automated-publishing)

## Overview

Civic provides a developer-friendly plug and play React Component as an easy way to request a Gateway Token from your dApp with minimal setup. It includes a Civic Pass status widget, the Identity Button; and an in-dApp modal to guide your users through the process.

## Official Documentation

You can find the official documentation hosted on [Gitbook](https://docs.civic.com/).

## Example Project

We have also created an example React project that shows how you would go about integrating the different features provided by our `@civic/solana-gateway-react` component project on [Github](https://github.com/civicteam/civic-pass-sol-template).

## Getting started

Please refer to the project on [Github](https://github.com/civicteam/civic-pass-sol-template) for a complete example.

### 1. Install the component

```bash
npm i @civic/solana-gateway-react
```

or using `yarn`

```bash
yarn add @civic/solana-gateway-react
```

### 2. Include the gateway context
Surround any code that needs access to the gateway token with:

```typescript jsx
import { GatewayProvider } from "@civic/solana-gateway-react";
import { Connection, clusterApiUrl } from '@solana/web3.js';

<GatewayProvider
  connection={new Connection(clusterApiUrl("mainnet-beta"), 'processed')}
  cluster="mainnet-beta"
	wallet={wallet}
	gatekeeperNetwork={gatekeeperNetwork}>
</GatewayProvider>

```

where:
- `connection` is a Solana connection to any Solana network, recommended commitment of 'processed'
- `cluster` is the Solana network to use, i.e. devnet, mainnet-beta, testnet. This defaults to mainnet-beta, so should be set if a different connection endpoint is used than mainnet-beta
- `wallet` contains `publicKey`, `connected` and `signTransaction` (see below)
- `gatekeeperNetwork` is a Solana publicKey provided by the dApp, you can get choose a gatekeeper network to use from https://docs.civic.com/integration-guides/civic-idv-services/available-networks


### 3. Accessing or requesting the gateway token
The component will automatically look for a gateway token on chain. To access it once created:

```typescript
import { useGateway } from "@civic/solana-gateway-react";
const { gatewayToken } = useGateway()
```

If the wallet does not have a gateway token, request it using `requestGatewayToken`:

```typescript
const { requestGatewayToken } = useGateway()
```

For example, by connecting it to a button component:

```typescript jsx
<button onclick={requestGatewayToken}>Validate your wallet</button>
```

Or by using Identity Button provided:

```typescript jsx
import IdentityButton from './lib/button/IdentityButton';
...
<IdentityButton />

```

### 4. IdentityButton behaviour

The IdentityButton is a reference implementation of a UI component to communicate to your dApp users the current status of their Gateway Token, or Gateway Token request. It changes appearance with text and icons to indicate when the user needs to take action and can be clicked by the user at any point in the process. The initial click for a new user will initiate the flow for the user to create a new Gateway Token. Once the user has gone through KYC and submitted their Gateway Token request via the Civic compliance iFrame, any subsequent click will launch the Civic Pass iframe with a screen describing the current status of the flow.


## API Documentation

### GatewayProvider
The `GatewayProvider` is a React component that gives children access to the GatewayContext through the `useGateway` function. This component holds the state for a given gateway token or gateway token request.

```typescript

export type GatewayProviderProps = {
  // The wallet connected to the dApp
  wallet: WalletAdapter;

  // The gatekeeper network public key
  gatekeeperNetwork: PublicKey;

  // A Solana connection to any Solana network (e.g. mainnet-beta, devnet, testnet)
  connection: Connection;

  // [Optional] Set Civic gatekeeper stage (testing only), defaults to 'prod'
  stage?: string;


  // [Optional] The cluster name. Defaults to mainnet-beta. Should be set if a different connection other than mainnet-beta is used
  cluster?: string;

  // [Optional] A react element that the dApp can wrap the iframe in to allow customer dApp styling
  wrapper?: React.FC;

  // [Optional] The url of your logo that will be shown, if set, during verification
  logo?: string;

  // [Optional] A redirect URL that can be used for deep-linking and mobile-web
  redirectUrl?: string;

  // [Optional] If this is set, the transaction must be signed by this payer, and the payer will
  // be charged the cost of the pass.
  // Leave unset (default) for the user to pay for the pass.
  // If set, you must also set handleTransaction. See an example below.
  payer?: PublicKey;

  // [Optional] A callback that will invoked with a partially signed transaction for the user or payer to sign and send to the blockchain.
  // Use this if you want to send the transaction to a backend to sign and send, or if you want to otherwise control how the transaction is signed and sent.
  handleTransaction?: (transaction: Transaction) => Promise<void>;

  // [Optional] If set, the gatekeeper will send the transaction to the blockchain. Defaults to false.
  // When true, `payer` must be false or unset, and `handleTransaction` must be unset.
  // Note: This is only supported for custom passes.
  gatekeeperSendsTransaction?: boolean;

  // [Optional] A set of client options (see 'Client Options' below)
  options?: Options;

  // [Optional] For passes with expiry, set this value to prompt the user to refresh the pass this amount of seconds before the pass expires
  expiryMarginSeconds?: number

  // [Optional] When set to true, initiates the refresh flow if the user has an active token, even if the pass hasn't expired
  forceRequireRefresh?: boolean;
};

```

### GatewayStatus
The `GatewayStatus` is an enum that reveals the state of the requested Gateway Token.

```typescript
export enum GatewayStatus {
  UNKNOWN,
  CHECKING,
  NOT_REQUESTED,
  COLLECTING_USER_INFORMATION,
  PROOF_OF_WALLET_OWNERSHIP,
  IN_REVIEW,
  REJECTED,
  REVOKED,
  FROZEN,
  ACTIVE,
  ERROR,
  LOCATION_NOT_SUPPORTED,
  VPN_NOT_SUPPORTED,
  REFRESH_TOKEN_REQUIRED,
  VALIDATING_USER_INFORMATION,
  USER_INFORMATION_VALIDATED,
  USER_INFORMATION_REJECTED,
}
```

The behaviour of the React component depends on the status of the token. For example, if the token status is `NOT_REQUESTED` and the user triggers the react component, then the flow to collect the user's information and request a Civic Token will be started.

For other cases, like the token status `FROZEN`, triggering the React component opens a dialog explaining to the user the current status and offering him to reach out to Civic is he requires assistance.

The table bellow lists all status:

| Status        | Description | Behaviour when triggered
| ------------- | ------------------------------------------------------------ | -------------------------------------------------------
| `UNKNOWN`       | No user wallet is connected or no gatekeeper network set     | *None* |
| `CHECKING`      | The Identity Component is making the relevant requests to check the state of the token      | *None* |
| `NOT_REQUESTED` | A token has not been requested  | Opens the Civic Pass modal dialog and initiates the token request flow |
| `COLLECTING_USER_INFORMATION` | The Identity Component is in progress and can be resumed | Opens the Civic Pass modal dialog and resumes the token request flow|
| `IN_REVIEW` | The token has been requested and the Gatekeeper is reviewing the request | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `ACTIVE` | The token has been issued successfully and is active. | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `FROZEN` | The token has been frozen, for example because the user connected from a blocked IP | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `REJECTED` | The token requests has been rejected by the Gatekeeper | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `REVOKED` | The token has been revoked, for example because the user connected from a banned IP | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `LOCATION_NOT_SUPPORTED` | The user's location is not currently supported | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `VPN_NOT_SUPPORTED` | The user is using a proxy or VPN | Opens the Civic Pass modal dialog with a user-friendly explanation of the status. |
| `REFRESH_TOKEN_REQUIRED` | The user needs to refresh their gateway token | Opens the Civic Pass modal dialog and takes the user through the flow for refreshing their token. |
| `ERROR` | There was an error retrieving the Gateway Token or completing a vrification flow | Opens the Civic Pass modal dialog and the user can restart the process. |
| `VALIDATING_USER_INFORMATION` | The validation process is currently being reviewed | Opens the Civic Pass modal dialog and shows a user friendly message |
| `VALIDATION_PROCESS_IN_PROGRESS` | The validation process is still in progress | Opens the Civic Pass modal dialog and the user can resume the process |
| `USER_INFORMATION_VALIDATED` | The validation process has completed | Opens the Civic Pass modal dialog and completes the process |
| `USER_INFORMATION_REJECTED` | The validation process was rejected | Opens the Civic Pass modal explaing the reasons for the failure |

### WalletAdapter
The wallet passed to the GatewayProvider must implement the WalletAdapter interface. By providing a publicKey (this will be linked with the GatewayToken), and a 'sign' function to prove wallet ownership:

```typescript
export interface WalletAdapter {
  publicKey?: PublicKey;
  signTransaction: (transaction: Transaction) => Promise<Transaction>;
  connected?: boolean;
}
```

### useGateway

Any component wrapped by `GatewayProvider` can access the state and useful functions of the GatewayProvider through this function.

Returns the current context values for the `GatewayContext` by exposing the following properties.

```typescript
export type GatewayProps = {
  requestGatewayToken: () => Promise<void>, // starts off gateway token process
  reinitialize: () => Promise<void> // reinitializes the GatewayClient instance
  gatewayStatus: GatewayStatus, // normally a value from a React hook state, defaults to GatewayStatus.UNKNOWN
  gatewayToken?: GatewayToken, // the current GatewayToken used in the dApp
  gatewayTokenTransaction: string, // if broadcastTransaction is false, this will be populated with any transactions generated by the backend
  pendingRequests?: PendingPayload; // an object containing ID(s) that a partner needs to resolve before a pass can be issued
}

const gatewayProps: GatewayProps = useGateway();
```

### Client options

You can specify some options that affect the display behaviour of the Civic modal that the user interacts with:

```typescript
export type Options = {
  // whether the Civic modal should appear automatically if the Civic Pass token state changes
  autoShowModal: boolean; 

  // debug | info | warn | error
  logLevel: LogLevel; 
};
```

### Triggering early pass refresh
If you want to force a user to refresh their pass before the pass has expired, the boolean value 'forceRequireRefresh' can be passed to the GatewayProviderProps, e.g. if you want the user to have to refresh if their pass is older than 1 day old:
```
const shouldForceRefresh = gatewayTokenExpiry < 1_DAY_FROM_NOW

<GatewayProvider wallet={...} gatekeeperNetwork={...} forceRequireRefresh={shouldForceRefresh}/>
```
This would cause the refresh flow to be initiated for a user with a pass that was created more than one day ago.

### Paying for your customers' passes

By default, the user requesting the pass will pay transaction costs and the cost of the pass.

If you want to pay for the pass, you can set the `payer` prop in the `GatewayProvider` to the address of the payer.

You must also set `handleTransaction`.

For example, to pay for the pass using a backend service:

```tsx
const handleTransaction = async (transaction: Transaction):Promise<string> => {
  // serialize the transaction to send to your backend. 
  const serializedTransaction = Buffer.from(transaction.serialize({
    requireAllSignatures: false,
  })).toString('base64');

  // call your backend to sign and send the transaction
  const response = await fetch('https://your-backend.com/sign-transaction', {
    method: 'POST',
    headers: { 'Content-Type': 'application/json' },
    body: JSON.stringify({ transaction: serializedTransaction }),
  });
  
  // return the tx signature
  const { signature } = await response.json();
  return signature;
};

return <GatewayProvider
  gatekeeperNetwork={GATEKEEPER_NETWORK}
  wallet={wallet}
  connection={connection}
  payer={payer}
  handleTransaction={handleTransaction}
>...</GatewayProvider>
```

Your backend code is up to you, but should look something like this (Warning - sample code - not for production):

```typescript
import { TransactionRequest, Wallet } from 'ethers';

// Must match the public key set as the payer in the frontend
const keypair = /* your keypair */;

app.post('/sign-transaction', async (req: Request<{}, {}, { transction: string }>, res: Response) => {
  // parse the request into a transaction
  const { transaction } = await request.json();
  const parsedTransaction = Transaction.from(Buffer.from(transaction, 'base64'));

  // sign the transaction
  // WARNING - Do NOT sign arbitrary transactions sent from an unsecured client. Your funds may be at risk.
  // This code is for demonstration purposes only
  parsedTransaction.partialSign(keypair);

  // send the transaction.
  const signature = await connection.sendRawTransaction(parsedTransaction.serialize(), {
    preflightCommitment: "processed"
  });

  // wait for the transaction to confirm
  const blockhash = await connection.getLatestBlockhash();
  await connection.getSignatureStatuses({ signature, ...blockhash });

  // return the tx signature
  res.json({ signature });
});
```

### Wrapper

You can customise how the verification flow is displayed by providing a custom wrapper.

```typescript
...
const customWrapperStyle: CSS.Properties = {
  backgroundColor: 'rgba(0,0,0,1)',
  position: 'fixed',
  zIndex: 999,
  width: '100%',
  height: '100%',
  overflow: 'auto',
  paddingTop: '5%'
}

const customWrapperContentStyle: CSS.Properties = {
  backgroundColor: '#fefefe',
  margin: 'auto',
  width: '90%',
  position: 'relative',
}

export const CustomWrapper: React.FC = ({ children = null }) => {
  return (
    <div style={customWrapperStyle}>
      <div style={customWrapperContentStyle}>
        <img style={{ maxWidth: '20%' }} src={logo} className="app-logo" alt="logo"/>
        {children}
      </div>
    </div>
  )
}

```

## Contributing

### `yarn build`

Builds the app for production to the `build` folder.
It correctly bundles React in production mode and optimizes the build for the best performance.

The build is minified and the filenames include the hashes.\
Your app is ready to be deployed!

### `yarn publish`

Publishes a new version of the React Component. We use beta releases before releasing an official release in the following format `(major).(minor).(patch)-beta.(build number)`. You can release a beta with the following command `yarn add @civic/solana-gateway-react@beta`.

### Automated publishing

To publish a new version of the react-component, add a Git tag and use a prefix along with the desired version number e.g.
```
solana-rc-0.7.0
```
Note that this method should only be used for production versions, not beta tags.
